use strict;
use Test::More tests => 29;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::String qw( QuoteXML UnquoteXML SplitEMailAddresses Slice Substr CompareStrings );
use Math::BigFloat;

sub Main {

    # QuoteXML

    ok( !defined QuoteXML(undef), "QuoteXML undef" );
    ok( defined QuoteXML(''), "QuoteXML ''" );
    ok( QuoteXML '<>&"\'' eq '&lt;&gt;&amp;&quot;&apos;', "QuoteXML '<>&\"\\''" );

    # UnquoteXML

    ok( !defined UnquoteXML(undef), "UnquoteXML undef" );
    ok( defined UnquoteXML(''), "UnquoteXML ''" );
    ok( UnquoteXML '&lt;&gt;&amp;&quot;&apos;' eq '<>&"\'', "UnquoteXML '<>&\"\\''" );

    # SplitEMailAddresses

    my %AddrListTests = (
        "acb;\"\\\\,\\\(\\\"\"cccc, \"rrr\"; a,  ; bd\\\\, abf\\\, (\",\\\\x); aaaddd <x\\\<xx\@h<hh,hhhh><hhhh>>,<sss>" =>
        [ "acb","\"\\\\,\\\(\\\"\"cccc","\"rrr\"","a","bd\\\\","abf\\","(\",\\\\x)","aaaddd <x\\\<xx\@h<hh,hhhh><hhhh>>","<sss>" ],
        '"Manuel Blechschmidt" <mblechschmidt@epages.de>;' =>
        [ '"Manuel Blechschmidt" <mblechschmidt@epages.de>' ],
        'mblechschmidt@epages.de;m.mustermann@epages.de' =>
        [ 'mblechschmidt@epages.de','m.mustermann@epages.de' ],
        '"Manuel Blechschmidt" <mblechschmidt@epages.de>; "Max; Mustermann" <m.mustermann@epages.de>' =>
        [ '"Manuel Blechschmidt" <mblechschmidt@epages.de>','"Max; Mustermann" <m.mustermann@epages.de>' ],
        '"O\'Neil\'s shop" <oneil@epages.de>; O\'Neil\'s shop <oneil@epages.de>' =>
        [ '"O\'Neil\'s shop" <oneil@epages.de>','O\'Neil\'s shop <oneil@epages.de>' ],
        '"Manuel Blechschmidt" <mblechschmidt@epages.de>; "John; \\"Mller" <j.doe@epages.de>' =>
        [ '"Manuel Blechschmidt" <mblechschmidt@epages.de>','"John; \\"Mller" <j.doe@epages.de>' ],
        '"<oneil@epages.de> O Neil\'s shop"; O Neil\'s shop <oneil@epages.de>' =>
        [ '"<oneil@epages.de> O Neil\'s shop"','O Neil\'s shop <oneil@epages.de>' ],
        ' oneil@epages.de;O N"eils" shop <mhoehn@epages.de>' =>
        [ 'oneil@epages.de','O N"eils" shop <mhoehn@epages.de>' ],
        );

    ok( $AddrListTests{$_} ~~ SplitEMailAddresses($_),  $_ ) foreach sort keys %AddrListTests;

    ok( Slice('This is a string to long to show in a navigation element', 15) eq 'This is a st...', "slice string with positive length and default postfix");
    ok( Slice('This is a string to long to show in a navigation element', 15, '***') eq 'This is a st***', "slice string with positive length and '***' as postfix");
    ok( Slice('This is a string to long to show in a navigation element') eq 'This is a string to long to...', "slice string with default length and default postfix");
    ok( Slice('This is a string to long to show in a navigation element', -15) eq '...tion element', "slice string with negative length and default postfix");

    ok( Substr('This is a string to long to show in a navigation element', 0, 15) eq 'This is a strin', "substring from first position to length");
    ok( Substr('This is a string to long to show in a navigation element', 0, -15) eq 'This is a string to long to show in a nav', "substring from first position to (length og string - 15)");
    ok( Substr('This is a string to long to show in a navigation element', 15) eq 'g to long to show in a navigation element', "substring with startposition 15");
    ok( Substr('This is a string to long to show in a navigation element', -15) eq 'igation element', "substring with startposition (length(String) -15)");
    ok( Substr('This is a string to long to show in a navigation element', 15, 15) eq 'g to long to sh', "substring with startposition (length(String) -15)");

    my @TestList = (
        {
            'Name'  => 'Wirkstoff: Aloe Vera, Swisa Beauty',
            'Title' => 'Wirkstoff: Aloe Vera, Swisa Beauty',
            'Ratio' => 1.0,
        },
        {
            'Name'  => 'Wirkstoff: Aloe Vera, Swisa Beauty',
            'Title' => 'Swisa Beauty - Aloe Vera - hautfreundliche Produkte',
            'Ratio' => 0.67,
        },
        {
            'Name'  => 'Wirkstoff: Aloe Vera, Swisa Beauty',
            'Title' => 'wirkstoffe: Aloe2 verAA, blah Swisas Beautye',
            'Ratio' => 0.83,
        },
        {
           'Name'  => 'Wirkstoff: Aloe Vera, Swisa Beauty',
           'Title' => 'Swisa Beauty - Aloe Vera - toll super kostenlos bestellen!',
           'Ratio' => 0.5,
        },
        {
            'Name'  => 'Wirkstoff: Aloe Vera, Swisa Beauty',
            'Title' => 'Sqisa Beetly - Quark Quatsch - toll super kostenlos bestellen!',
            'Ratio' => 0.12,
        },
    );
    foreach my $Item (@TestList) {
        my $Ratio = CompareStrings($Item->{'Title'}, $Item->{'Name'});
        my $RoundedRatio = Math::BigFloat->new($Ratio)->ffround(-2);
        cmp_ok( $RoundedRatio, '==', $Item->{'Ratio'}, 'CompareStrings: check if strings are similar. Returned ratio is ' . $RoundedRatio . '.' );
    }

    # coverage parameter
    my $Ratio = CompareStrings(
        'Wirkstoff: Aloe Vera, Swisa Beauty',
        'Sqisa Beetly - Quark Quatsch - toll super kostenlos bestellen!',
        0.5
    );
    my $RoundedRatio = Math::BigFloat->new($Ratio)->ffround(-2);
    cmp_ok($RoundedRatio, '==', 0.8, 'Run CompareStrings with coverage parameter. Returned ratio is ' . $RoundedRatio . '.');
}

RunScript(Sub=>\&Main);
