use Test::More tests => 16;
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath ExistsObjectByPath LoadRootObject);

use DE_EPAGES::FedEx::API::RemoteFedExBarcode;
use DE_EPAGES::FedEx::UI::Basket;
use DE_EPAGES::Customer::API::Object::Customer;
use DE_EPAGES::Order::API::Object::Basket;

use strict;

my $ROOTPATH = '/Test';

sub run_tests {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # initializsation
            ok( Init(), 'initialization' );
            eval {
                testImport();
                testBarCodeCreation();
                #testRealTimeCalculation();
            };
            if (ExistsError()) {
                my $Error = GetError();
                fail('Error - '. $Error->code);
                diag($Error->longMessage);
                # always cleanup
                ok( Cleanup(), 'cleanup');
                Error($Error);
            }
            # cleanup
            ok( Cleanup(), 'cleanup');
        }
    );
}

sub testImport{
    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShopFedEx');
    isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/FedEx');
    isa_ok($ShippingFulfillmentMethod, 'DE_EPAGES::FedEx::API::Object::ShippingFulfillmentMethodFedEx');


    my $Customer = $Shop->path('Customers/TestCustomer');
    isa_ok($Customer, 'DE_EPAGES::Customer::API::Object::Customer' );

    my $Order = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    isa_ok($Order, 'DE_EPAGES::Order::API::Object::CustomerOrder' );

    cmp_ok(scalar @{$Order->get('Documents')}, '>=',1, 'number of orderdocuments');

    my $ShippingLabelFedEx = $Shop->path('Customers/TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');
    isa_ok($ShippingLabelFedEx, 'DE_EPAGES::FedEx::API::Object::ShippingLabelFedEx' );

    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderFedEx');
    isa_ok($Transaction, 'DE_EPAGES::ExternalShipping::API::Object::ShippingTransaction' );

    my $Basket = DE_EPAGES::Order::API::Object::Basket->new();
    isa_ok($Basket, 'DE_EPAGES::Order::API::Object::Basket' );

    cmp_ok(scalar @{$Transaction->get('ShippingLabels')}, '>=',1, 'number of labels');

    foreach my $ShippingLabel (@{$Transaction->get('ShippingLabels')}) {
        isa_ok($ShippingLabel, 'DE_EPAGES::FedEx::API::Object::ShippingLabelFedEx' );
    }
}

sub testBarCodeCreation {
    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShopFedEx');
    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/FedEx');
    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderFedEx');

    my $Remote = DE_EPAGES::FedEx::API::RemoteFedExBarcode->new();
    my $cont = 0;
    foreach my $ShippingLabel (@{$Transaction->get('ShippingLabels')})
    {   
        $cont++; 
        my $result = $Remote->ProcessShipmentRequest($ShippingFulfillmentMethod,$ShippingLabel,'PDF',1);
        cmp_ok(isValid( $result->{'Image'} ),'==',1,'label is returned');
    }
    cmp_ok($cont, '==',2, 'number of returned items');
}

sub testRealTimeCalculation {
    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShopFedEx');
    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/FedEx');
    
    my $Basket = DE_EPAGES::Order::API::Object::Basket->new();
    isa_ok($Basket, 'DE_EPAGES::Order::API::Object::Basket' );
   
    my $Customer = $Shop->path('Customers/TestCustomer');
    isa_ok($Customer, 'DE_EPAGES::Customer::API::Object::Customer' );
    
    my $User = $Shop->path('Customers/TestCustomer/Users/mmustermann');
    isa_ok($User, 'DE_EPAGES::Permission::API::Object::User' );
    
    my $WeightDimensions = {
        'Weight' => '2',
        'Height' => '2',
        'Width' => '2',
        'Length' => '2'
    }; 

    my $BasketObj = DE_EPAGES::FedEx::UI::Basket->new();

    my $result = $BasketObj->ProcessRateRequest($ShippingFulfillmentMethod, 1,$Basket,$User,$WeightDimensions);
    cmp_ok(isValid( $result->{'ShippingFulfillmentProduct'} ),'==',1,'Services costs are returned');
}
sub Main {
    run_tests;
    done_testing;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript( Sub => \&Main );

1;