#========================================================================================
# package      DE_EPAGES::RemoteSearch::t::indexIntegration
# state        private
# description  Index integration tests
#========================================================================================
package DE_EPAGES::RemoteSearch::t::indexIntegration;
use base Test::Class;

use strict;
use Test::More;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
    JoinObjectPath
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw (ImportAttributes);
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw ( GetOrCreateSearchIndex DropAllSearchIndexes );
use DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder;
use DE_EPAGES::RemoteSearch::API::IndexStatus qw ( 
    INDEXSTATUS_PENDING
    INDEXSTATUS_COMPLETE
    INDEXSTATUS_OFF
    IndexStatusString
);

use URI;
use LWP::UserAgent;
use HTTP::Request::Common qw ( GET );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::File qw ( RunOnDirectory );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use File::Basename;

my $ROOTALIAS = 'Test';
my $ROOTPATH = JoinObjectPath(['',$ROOTALIAS]);

sub checkPreconditions {
    my $class = shift;
    if (! LoadRootObject->get('RemoteSearchEnabled')) {
        $class->SKIP_CLASS('remote search not enabled');
    }
}

sub init : Test(startup) {
    my $self = shift;
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    $self->cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => $ROOTALIAS } );
    ImportObjects( 'import.xml', $ROOTPATH );
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode('en');
    ImportAttributes( 'import_en.xml', $ROOTPATH );
    return 1;
}

sub cleanup : Test(shutdown) {
    my $self = shift;
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
}

sub SetUp : Test(setup) {
    my $self = shift;
    $self->{Shop}  = LoadObjectByPath(JoinObjectPath(['',$ROOTALIAS, 'TestShopRemoteSearch']));
    $self->{Index} = GetOrCreateSearchIndex($self->shop);
    my $Url = 'http://localhost:8088/rs/';
    $self->index()->set({ 'Logging' => 1, RemoteSearchURL => $Url });

    Error ('Could not ping Host: ' . URI->new($Url)->host)
        unless $self->index()->remoteIndex->ping()
}

sub testInitializeIndex : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
}

sub testDropSearch : Test(no_plan) {
    my $self = shift;
    $self->index->drop();
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_OFF, 'testDropSearch: index off');
    $self->checkForTaskCompletion();
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_OFF, 'testDropSearch: index off')
    or diag($self->index()->get('UpdateTaskURL'));
}

sub testDropAllSearchIndexes : Test(no_plan) {
    my $self = shift;
    DropAllSearchIndexes($self->shop);
    ok(!$self->shop->existsChild('SearchIndexes'), 'reset removes all search indexes');
}

sub testUpdateIndexItems : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    $self->index()->updateItems($self->shop->get('Products'));
    $self->checkForTaskCompletion();
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testUpdateIndexItems: index clean')
    or diag($self->index()->get('UpdateTaskURL'));
    is($self->index()->get('IndexStatusString'), INDEXSTATUS_COMPLETE, 'check read only attribute IndexStatusString');
}

sub testUpdateIndexItemsByHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    $self->shop->folder('Products')->child('cg_0101104470')->set({ 'Name' => 'My New Product Name'}, 1);
    $self->checkForTaskCompletion();
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testUpdateIndexItemsByHook: index clean')
    or diag($self->index()->get('UpdateTaskURL'));
}

sub testDeleteIndexItemsByHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    $self->shop->folder('Products')->child('cg_0101104471')->delete;
    $self->checkForTaskCompletion();
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testDeleteIndexItemsByHook: index clean')
    or diag($self->index()->get('UpdateTaskURL'));
}

sub testInsertIndexItemsByHook : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    $self->shop->folder('Products')->insertChild('Product', {Alias => 'xx_0101104472'});
    $self->checkForTaskCompletion();
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testInsertIndexItemsByHook: index clean')
    or diag($self->index()->get('UpdateTaskURL'));
}

sub testQuerySuggest : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    my $hResult = $self->index->remoteSuggest->suggestAsHash('Deuter','de');
    is($hResult->{manufacturers}->[0]->{name},'Deuter','testQuerySuggest: suggest contains Deuter.');
}

sub testDeleteIndexItems : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    $self->index()->deleteItems($self->shop->get('Products'));
    $self->checkForTaskCompletion();
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'testDeleteIndexItems: index clean')
    or diag($self->index()->get('UpdateTaskURL'));
}

sub testSimpleSearch : Test(no_plan) {
    my $self = shift;
    $self->initializeIndex;
    my $hParams = DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder->new()->lang('en')->query('Grangers')->params();
    my $Response = $self->index->remoteSearch->searchAsObject($hParams);
    is($Response->results, 1, 'testSimpleSearch: Grangers has one result');
    is($Response->items->[0]->{'filter'}
        , $self->shop->folder('Products')->child('ht_0801099030')->id
        , 'testSimpleSearch: Grangers ObjectID matches');
}

sub initializeIndex {
    my $self = shift;
    return if (IndexStatusString($self->index()->get('IndexStatus')) eq INDEXSTATUS_COMPLETE);
    GetOrCreateSearchIndex($self->shop)->initialize();
    my $TaskURL = $self->index->get('UpdateTaskURL');
    like($TaskURL, qr|http://\w+:\d+/rs|, 'task url');
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_PENDING, 'initializeIndex: pending operation');
    $self->checkForTaskCompletion();
    is(IndexStatusString($self->index()->get('IndexStatus')), INDEXSTATUS_COMPLETE, 'initializeIndex: index ready')
    or diag($self->index()->get('UpdateTaskURL'));
}

sub checkForTaskCompletion {
    my $self = shift;
    for ( my $i = 0; $i< 9; ++$i) {
        sleep 1;
        $self->index->updateIndexStatus($self->index()->getUpdateTask);
        last if (IndexStatusString($self->index()->get('IndexStatus')) eq INDEXSTATUS_COMPLETE);
    }
    # hack: make sure the changes have been committed to solr index.
    my $ua = LWP::UserAgent->new;
    my $Request = GET('http://localhost:8983/solr/Products/update/?commit=true');
    $ua->request($Request);
}

sub shop {
    return shift->{Shop};
}

sub index {
    return shift->{Index};
}

####### Main ############
RunScript(
    Sub => sub {
        RunOnDirectory(
            dirname(__FILE__),
            sub {
                RunOnStore(
                    'Store' => GetTestSite(),
                    'Sub'   => sub {
                        DE_EPAGES::RemoteSearch::t::indexIntegration->checkPreconditions();
                        Test::Class->runtests;
                    }
                );
            }
        );
    }
);
