use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError Error );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Object::API::Factory qw(LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Core::API::PerlTools qw( GetLocalTimeZone );
use DE_EPAGES::SEUR::API::RemoteSEURTransaction;


use strict;

my $ROOTPATH = '/Test';


sub run_tests {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # initializsation
            ok( Init(), 'initialization' );
            eval {
                my ($continue, $hObjects) = testImport();
                return unless ( $continue );

                $hObjects->{'remoteTransaction'} = DE_EPAGES::SEUR::API::RemoteSEURTransaction->new($hObjects->{'shipFM'}, $hObjects->{'transaction'});

                my $aShippingLabels = testPublicService($hObjects);
                return unless ( $aShippingLabels );

                my $hShipmentsByDate = testShipmentService($hObjects, $aShippingLabels);

                testPickupService($hObjects, $hShipmentsByDate);

                testExpeditionService($hObjects);
            };
            if (ExistsError()) {
                my $Error = GetError();
                fail('Error - '. $Error->code);
                diag($Error->longMessage);
                # always cleanup
                ok( Cleanup(), 'cleanup');
                Error($Error);
            }
            # cleanup
            ok( Cleanup(), 'cleanup');
        }
    );
}


sub testImport{
    my $testImportOK = 0;

    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShopSEUR');
    $testImportOK = isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/SEURLogistics');
    $testImportOK = isa_ok($ShippingFulfillmentMethod, 'DE_EPAGES::SEUR::API::Object::ShippingFulfillmentMethodSEUR') && $testImportOK;

    my $Customer = $Shop->path('Customers/TestCustomer');
    $testImportOK = isa_ok($Customer, 'DE_EPAGES::Customer::API::Object::Customer' ) && $testImportOK;

    my $Order = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    $testImportOK = isa_ok($Order, 'DE_EPAGES::Order::API::Object::CustomerOrder' ) && $testImportOK;

    $testImportOK = cmp_ok(scalar @{$Order->get('Documents')}, '>=',1, 'number of orderdocuments') && $testImportOK;

    my $ShippingLabel = $Shop->path('Customers/TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');
    $testImportOK = isa_ok($ShippingLabel, 'DE_EPAGES::SEUR::API::Object::ShippingLabelSEUR' ) && $testImportOK;

    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderSEUR');
    $testImportOK = isa_ok($Transaction, 'DE_EPAGES::ExternalShipping::API::Object::ShippingTransaction' ) && $testImportOK;

    $testImportOK = cmp_ok(scalar @{$Transaction->get('ShippingLabels')}, '>=',1, 'number of labels') && $testImportOK;

    foreach my $ShippingLabel (@{$Transaction->get('ShippingLabels')}) {
        $testImportOK = isa_ok($ShippingLabel, 'DE_EPAGES::SEUR::API::Object::ShippingLabelSEUR' ) && $testImportOK;
    }
    return ($testImportOK, {    'shop'          => $Shop,
                                'shipFM'        => $ShippingFulfillmentMethod,
                                'customer'      => $Customer,
                                'order'         => $Order,
                                'transaction'   => $Transaction,
    });
}

sub testPublicService{
    my ($hObjects) = @_;

    my $RemoteTransaction = $hObjects->{'remoteTransaction'};
    my $Transaction = $hObjects->{'transaction'};
    my $Shop = $hObjects->{'shop'};
    my $Order = $hObjects->{'order'};

    # test shop address against SEUR public web service
    my ($locShopError) = $RemoteTransaction->wssetShopLocation($Shop->get('Address'));
    cmp_ok($locShopError,'!=',1,'Shop zipcode ' . $Shop->get('Address')->get('Zipcode'));

    # test shipping label addresses against SEUR public web service
    my @aValidShippingLabels = ();
    my $ShippingLabels = $Transaction->get('ShippingLabels');

    foreach my $ShippingLabel (@$ShippingLabels) {
        my ($numLocations, $zipCode, $location, $province, $countryCode) = $RemoteTransaction->wsgetLocation($Order->get('ShippingAddress'));
        if ( cmp_ok($numLocations,'>=',1,'Customer zipcode ' . $zipCode . ' on shipping label ' . $ShippingLabel->get('Alias')) ) {
            push (@aValidShippingLabels, {  'Label'         => $ShippingLabel,
                                            'Zipcode'       => $zipCode,
                                            'Location'      => $location,
                                            'Province'      => $province,
                                            'Countrycode'   => $countryCode,
            });
        }
    }
    return \@aValidShippingLabels;
}

sub testShipmentService{
    my ($hObjects, $aValidShippingLabels) = @_;

    my $RemoteTransaction = $hObjects->{'remoteTransaction'};

    my $hValidShippingLabelsByDate = {};
    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();

    foreach my $hShipment (@$aValidShippingLabels) {
        my ($shipError, $hResult) = $RemoteTransaction->wssetShipment($hShipment);
        my $ShippingLabel = $hShipment->{'Label'};
        cmp_ok($shipError,'!=', 1, 'Shipment on shipping label ' . $ShippingLabel->get('Alias'));

        if (!$shipError) {
            # set not outdated pickup date
            my $pickupDate = DateTime->now(time_zone => GetLocalTimeZone());
            if ( $pickupDate->day_of_week_0() < 4 ) {
                $pickupDate->add( days => 1 );
            } else {
                $pickupDate->add( days => 3 );
            }
            $ShippingLabel->set({ 'PickupDate' => $pickupDate });

            my $strDate = $Formatter->format_datetime($ShippingLabel->get('PickupDate'), 'date', '%Y-%m-%d');
            $hValidShippingLabelsByDate->{$strDate} = [] unless defined($hValidShippingLabelsByDate->{$strDate});
            my $aPointer = $hValidShippingLabelsByDate->{$strDate};
            push (@$aPointer, { 'Label'         => $ShippingLabel,
                                'ECB'           => $hResult->{'ECB'},
                                'Zipcode'       => $hShipment->{'Zipcode'},
                                'Location'      => $hShipment->{'Location'},
                                'Province'      => $hShipment->{'Province'},
                                'Countrycode'   => $hShipment->{'Countrycode'},
            });
        }
    }
    return $hValidShippingLabelsByDate;
}

sub testPickupService{
    my ($hObjects, $hShippingLabelsByDate) = @_;

    my $RemoteTransaction = $hObjects->{'remoteTransaction'};

    foreach my $strPickupDate (keys %$hShippingLabelsByDate) {
        my $aShippingLabels = $hShippingLabelsByDate->{$strPickupDate};
        my ($pickError) = $RemoteTransaction->wssetPickup($strPickupDate, $aShippingLabels);
        cmp_ok($pickError,'!=',1,"Pickup on date: $strPickupDate");
    }
}

sub testExpeditionService{
    my ($hObjects) = @_;

    my $RemoteTransaction = $hObjects->{'remoteTransaction'};

    my $today = DateTime->today(time_zone => GetLocalTimeZone());
    my ($expError) = $RemoteTransaction->wssetExpedition($today);
    cmp_ok($expError,'!=',1,'Expedition PDF document');
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}


sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}



sub Main {
    run_tests;
    done_testing;
}


RunScript( Sub => \&Main );

1;