############################################################################
#
#   BatchProcessHandler.t
#
#   BatchProcessHandler is abstract, hence we need to subclass it. For
#   testing of processItems(), processItemList() a simple Item class
#   with no real functionality is created
#
############################################################################
use strict;

## no critic (DE_EPAGES::Style::RequireCommentForExportedFunctions)
## no critic (Modules::ProhibitMultiplePackages)
package My::API::Object::ExampleBatchHandler;
use parent qw( DE_EPAGES::Trigger::API::Object::BatchProcessHandler );

use DE_EPAGES::Core::API::Error qw (Error);

sub new {
    my $class = shift;
    my ($hOptions) = @_;

    return $class->SUPER::new($hOptions);
}

sub customOption {
    my $self = shift;
    return $self->{Options}->{CustomOption} if ( defined $self->{Options}->{CustomOption} );
}

sub processItem {
    my $self = shift;
    my ($Item) = @_;

    # to keep it short we don't do anything usefull
    Error('No Valid ID for Item') if ( $Item->id() < 1 );
}

sub processItemList {
    my $self = shift;

    foreach my $Item ( @{ $self->{_Items} } ) {
        $self->processItem($Item);
    }

    $self->cleanupItems();
}

###############################################################################

package My::API::Object::Item;
use base qw ( DE_EPAGES::Core::API::Object::BaseObject );

sub id {
    shift->get('ID');
}

###############################################################################

package My::Test;

use Test::More tests => 15;
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError );

use DE_EPAGES::Trigger::API::Object::BatchProcessHandler;

use constant BATCH_SIZE => 3;

sub Main() {

    local $| = 1;
    Tests();
}

############################################
#                                          #
#             TESTS                        #
#                                          #
############################################

sub Tests {
    my $Handler = My::API::Object::ExampleBatchHandler->new();
    isa_ok( $Handler, 'DE_EPAGES::Trigger::API::Object::BatchProcessHandler', 'BatchProcessHandler' );
    isa_ok( $Handler, 'My::API::Object::ExampleBatchHandler', 'ExampleBatchHandler' );

    like($Handler->packageName(), qr/My::API::Object::ExampleBatchHandler/, 'packageName()');

    like ($Handler->items(), qr/ARRAY/, 'items()');

    is( $Handler->batchSize(), 0, 'default BatchSize' );

    is( $Handler->countItems(), 0, 'countItems() no items added' );

    ok($Handler->processItemList(), 'processItemList');

    eval{
        my $Item = My::API::Object::Item->new( ID => 8 );
        $Handler->processItem($Item); #usaly called by processItemList()
    };
    ok(!ExistsError, 'processItem') or diag (GetError->message);

    my $count = _addItems($Handler);    #add some items via addItem() method

    ok( $Handler, 'add items' );

    is( $Handler->countItems(), $count, 'eight items counted' );

    $Handler->processItemList();

    is( $Handler->countItems(), 0, 'items processed' );

    $Handler = My::API::Object::ExampleBatchHandler->new( { BatchSize => BATCH_SIZE, CustomOption => 'HaHaHa' } );
    ok( $Handler, 'set custom option with constructor' );

    like( $Handler->customOption, qr/HaHaHa/, 'getter for custom option' );

    is( $Handler->batchSize(), BATCH_SIZE, 'set/read BatchSize' );

    $count = _addItems($Handler); #add some items via addItem() method

    is(
        $Handler->countItems(),
        $count % BATCH_SIZE,
        sprintf(
            'added %d Items -> pocessed %d x %d items, remaining %d',
            $count,
            $count / BATCH_SIZE,
            BATCH_SIZE,
            $count % BATCH_SIZE
        )
    );

}

sub _addItems($) {
    my ($Handler) = @_;

    my $count;
    foreach (qw(1337 42 23 13 7 16 52 2)) {
        my $Item = My::API::Object::Item->new( ID => $_ );
        $Handler->addItem($Item);
        $count++;
    }
    return $count;
}

RunScript( Sub => sub { Tests() } );
